exception Unknow_type of string
exception TimedOut
exception Malformed of string
type classtype = IN | CS | CH | HS
and rrtype =
    A of string
  | AAAA of string
  | AXFR
  | CNAME of string
  | HINFO of string * string
  | MX of int * string
  | NS of string
  | NULL of string
  | PTR of string
  | SOA of string * string * int * int * int * int * int
  | TXT of string
  | Error
  | NotFound
and dnstree = DomainName of string * dnstree list * rrtype list
class dns_auth :
  object
    method add : string -> rrtype list -> unit
    method private build_list : string list -> string list
    method clear : unit
    method get : string -> rrtype list
    method get_domain : string -> string
    method get_subtree : string -> dnstree list
    method is_authoritative : string -> bool
    method replace : string -> rrtype list -> unit
    method set : dnstree list -> unit
    method private str2list : string -> string list
    val mutable root : dnstree list
  end
class dns_data_header :
  object
    method get_ancount : int
    method get_flags : int
    method get_id : int
    method get_naddrr : int
    method get_nscount : int
    method get_opcode : int
    method get_qdcount : int
    method get_rcode : int
    method get_z : int
    method is_aa : bool
    method is_qr : bool
    method is_ra : bool
    method is_rd : bool
    method is_tc : bool
    method output : string
    method set_aa : bool -> unit
    method set_ancount : int -> unit
    method set_flags : int -> unit
    method set_id : int -> unit
    method set_naddrr : int -> unit
    method set_nscount : int -> unit
    method set_opcode : int -> unit
    method set_qdcount : int -> unit
    method set_qr : bool -> unit
    method set_ra : bool -> unit
    method set_rcode : int -> unit
    method set_rd : bool -> unit
    method set_tc : bool -> unit
    method set_z : int -> unit
    val mutable ancount : int
    val mutable flags : int
    val mutable id : int
    val mutable naddrr : int
    val mutable nscount : int
    val mutable qdcount : int
  end
class dns_data_question :
  object
    method get_class : int
    method get_name : string
    method get_type : int
    method output : string
    method private prepare_name : string -> string
    method set_class : int -> unit
    method set_name : string -> unit
    method set_type : int -> unit
    val mutable classs : int
    val mutable name : string
    val mutable typee : int
  end
class dns_data_answer :
  object
    method private compute_rdlength : unit
    method get_class : int
    method get_name : string
    method get_rdata : rrtype
    method get_rdlength : int
    method get_ttl : int
    method get_type : int
    method output : string
    method private output_rrtype : rrtype -> string
    method private prepare_address : string -> string
    method private prepare_name : string -> string
    method set_class : int -> unit
    method set_name : string -> unit
    method set_rdata : rrtype -> unit
    method set_rdlength : int -> unit
    method set_ttl : int -> unit
    method set_type : int -> unit
    val mutable classs : int
    val mutable name : string
    val mutable rdata : rrtype
    val mutable rdlength : int
    val mutable ttl : int
    val mutable typee : int
  end
class dns_packet :
  object
    method add_additionnal : dns_data_answer list -> unit
    method add_answer : dns_data_answer list -> unit
    method add_authority : dns_data_answer list -> unit
    method add_question : dns_data_question list -> unit
    method get_additionnals : dns_data_answer list
    method get_ancount : int
    method get_answers : dns_data_answer list
    method get_authorities : dns_data_answer list
    method get_flags : int
    method get_header : dns_data_header
    method get_id : int
    method get_naddrr : int
    method get_nscount : int
    method get_opcode : int
    method get_qdcount : int
    method get_questions : dns_data_question list
    method get_rcode : int
    method get_z : int
    method is_aa : bool
    method is_qr : bool
    method is_ra : bool
    method is_rd : bool
    method is_tc : bool
    method private list_of_questions :
      dns_data_answer list -> dns_data_question list
    method output : string
    method private output_list : dns_data_question list -> string
    method set_aa : bool -> unit
    method set_ancount : int -> unit
    method set_flags : int -> unit
    method set_header : dns_data_header -> unit
    method set_id : int -> unit
    method set_naddrr : int -> unit
    method set_nscount : int -> unit
    method set_opcode : int -> unit
    method set_qdcount : int -> unit
    method set_qr : bool -> unit
    method set_ra : bool -> unit
    method set_rcode : int -> unit
    method set_rd : bool -> unit
    method set_tc : bool -> unit
    method set_z : int -> unit
    val mutable additionnals : dns_data_answer list
    val mutable ancount : int
    val mutable answers : dns_data_answer list
    val mutable authorities : dns_data_answer list
    val mutable flags : int
    val mutable id : int
    val mutable naddrr : int
    val mutable nscount : int
    val mutable qdcount : int
    val mutable questions : dns_data_question list
  end
class dns_packet_parser :
  object
    method private add_string : int * string * int -> unit
    method private get_name : int -> string
    method parse_addr : char Stream.t -> string
    method parse_addr6 : char Stream.t -> string
    method parse_answer : char Stream.t -> dns_data_answer
    method parse_char_string : char Stream.t -> string
    method parse_header : char Stream.t -> dns_data_header
    method parse_name : int -> char Stream.t -> string
    method parse_packet : char Stream.t -> dns_packet
    method parse_query : char Stream.t -> dns_data_question
    val mutable table : (int * string * int) list
  end
class virtual dns_service :
  string ->
  object
    method virtual action : unit
    method private add_string : int * string * int -> unit
    method get_answer : dns_packet
    method private get_name : int -> string
    method get_port : int
    method get_query : dns_packet
    method get_server : string
    method get_timeout : float
    method parse_addr : char Stream.t -> string
    method parse_addr6 : char Stream.t -> string
    method parse_answer : char Stream.t -> dns_data_answer
    method parse_char_string : char Stream.t -> string
    method parse_header : char Stream.t -> dns_data_header
    method parse_name : int -> char Stream.t -> string
    method parse_packet : char Stream.t -> dns_packet
    method parse_query : char Stream.t -> dns_data_question
    method receive : Unix.file_descr -> unit
    method send : Unix.file_descr -> unit
    method set_answer : dns_packet -> unit
    method set_port : int -> unit
    method set_query : dns_packet -> unit
    method set_server : string -> unit
    method set_timeout : float -> unit
    val mutable answer : dns_packet
    val mutable port : int
    val mutable query : dns_packet
    val mutable server : string
    val mutable table : (int * string * int) list
    val mutable timeout : float
  end
class dns_query :
  string ->
  object
    method action : unit
    method add_query : string -> int -> int -> unit
    method private add_string : int * string * int -> unit
    method get_answer : dns_packet
    method private get_name : int -> string
    method get_port : int
    method get_query : dns_packet
    method get_server : string
    method get_timeout : float
    method parse_addr : char Stream.t -> string
    method parse_addr6 : char Stream.t -> string
    method parse_answer : char Stream.t -> dns_data_answer
    method parse_char_string : char Stream.t -> string
    method parse_header : char Stream.t -> dns_data_header
    method parse_name : int -> char Stream.t -> string
    method parse_packet : char Stream.t -> dns_packet
    method parse_query : char Stream.t -> dns_data_question
    method receive : Unix.file_descr -> unit
    method send : Unix.file_descr -> unit
    method set_answer : dns_packet -> unit
    method set_port : int -> unit
    method set_query : dns_packet -> unit
    method set_server : string -> unit
    method set_timeout : float -> unit
    val mutable answer : dns_packet
    val mutable port : int
    val mutable query : dns_packet
    val mutable server : string
    val mutable table : (int * string * int) list
    val mutable timeout : float
  end
class dns_axfr :
  string ->
  object
    method action : unit
    method private add_string : int * string * int -> unit
    method private break_stream : string -> unit
    method get_answer : dns_packet
    method private get_domain : string -> string
    method get_lst : dns_packet list
    method private get_name : int -> string
    method get_port : int
    method get_query : dns_packet
    method get_server : string
    method get_timeout : float
    method ls : string -> unit
    method parse_addr : char Stream.t -> string
    method parse_addr6 : char Stream.t -> string
    method parse_answer : char Stream.t -> dns_data_answer
    method private parse_axfr : char Stream.t -> dns_packet
    method parse_char_string : char Stream.t -> string
    method parse_header : char Stream.t -> dns_data_header
    method parse_name : int -> char Stream.t -> string
    method parse_packet : char Stream.t -> dns_packet
    method parse_query : char Stream.t -> dns_data_question
    method receive : Unix.file_descr -> unit
    method send : Unix.file_descr -> unit
    method set_answer : dns_packet -> unit
    method set_port : int -> unit
    method set_query : dns_packet -> unit
    method set_server : string -> unit
    method set_timeout : float -> unit
    val mutable answer : dns_packet
    val mutable leftstream : string
    val mutable lst : dns_packet list
    val mutable port : int
    val mutable query : dns_packet
    val mutable server : string
    val mutable soa : int
    val mutable table : (int * string * int) list
    val mutable timeout : float
  end
