(* faustine: resolver !
- finally get the good answer !
- add good tests ...
*)

exception AnswerFound;;
exception AnswerNotFound;;

class dns_resolver =
object (self)
    inherit Dns.dns_auth as super

    val mutable servers = [Dns.NS ("128.63.2.53");
			Dns.NS ("192.33.4.12");
			Dns.NS ("192.112.36.4");
			Dns.NS ("192.5.5.241");
			Dns.NS ("128.9.0.107");
			Dns.NS ("198.41.0.10")]

    method query str =
      (* query the cache *)
      let rsp = super#get str in
      if rsp = [Dns.NotFound]
      then
      (* let's get NS for str's domain ! *)
         (self#resolve str;
         let domain = List.fold_right (fun x -> fun y -> x^"."^y) (List.tl
         (Netstring_str.split (Netstring_str.regexp "\.") str)) "" in
         let srv = self#xts (List.hd (self#get_server domain))
         and qst = new Dns.dns_query "" in
         (*
         let ns = self#xta (List.hd (self#get srv)) in
         (* send the reply *)
         qst#set_server ns;
         qst#add_query str 1 1;
         qst#action
         *)
         ())
      else
      (* send the reply *)
         ()

    method private resolve str =
      let rq = Netstring_str.split (Netstring_str.regexp "\.") str in
      let queries = List.rev (List.tl (self#build_list rq)) in
      (* lookup each part of str "tld", "domain.tld", 
         "host.domain.tld", etc ... *)
      List.iter self#lookup queries (* error checking ? *)

    method private get_server domain =
      let rsp = super#get domain in
      if rsp != [Dns.NotFound]
      then
         self#find_ns rsp
      else
         rsp

    method private find_ns rsp =
      match rsp with
      | [] -> [Dns.NotFound]
      | (Dns.NS name)::l -> [Dns.NS name]
      | _::l-> self#find_ns l 

    method private lookup str =
    (* Beginnig of the resolver *)
    let domain = ref (List.fold_right (fun x -> fun y -> x^"."^y) (List.tl
    (Netstring_str.split (Netstring_str.regexp "\.") str)) "") in
    let qst = new Dns.dns_query "" in
    print_string ("Query: "^str^" - "^(!domain)^"\n"); flush stdout;
    (* Use all the servers we know !! *)
    try
    if !domain = "" || self#get_server !domain = [] || self#get_server !domain = [Dns.NotFound]
    then
	(print_string ("#get_root ->"^(!domain)^"<-\n"); flush stdout;
	qst#set_server self#get_root)
   else
      (print_string ("=>"^str^"\n"); flush stdout;
      let srv = self#xts (List.hd (self#get_server !domain)) in
      let ns = self#xta (List.hd (self#get srv)) in
      print_string ("Domain: "^(!domain)^" - "^ns^" ("^srv^")\n");flush stdout;
      qst#set_server ns);
    qst#set_timeout 5.;
    qst#add_query str 2 1; (* query NS rr for this zone *)
    (try
    qst#action;
    let respe = qst#get_answer#get_responses
    and adde = qst#get_answer#get_additionnals in
    (* verify type ! *)
    let grooar lst sch = 
	List.iter (fun x ->
	    if sch = x#get_name
	    then
		(
		print_string ("server>"^x#get_name^" "^str^" "^(self#xta x#get_rdata)^"\n");
		self#add str [Dns.NS (String.lowercase x#get_name)];
		self#add x#get_name [Dns.A (String.lowercase (self#xta x#get_rdata))];)
	    )
	lst 
    in
    List.iter (fun x -> grooar adde (self#xts x#get_rdata)) respe;
    raise AnswerFound
    with
    | Dns.TimedOut -> (); (* Let's do it again *)
    raise AnswerNotFound)
    with
    | AnswerFound -> () (* send the reply *)
    | AnswerNotFound -> () (* send a no such name *)

    method private get_root =
	let tmp1 = List.tl servers
	and tmp2 = List.hd servers in
	servers <- tmp1@[tmp2];
	match tmp2 with
	| Dns.NS a -> a
	| _ -> raise (Dns.Unknow_type "#get_server")

    method private build_list lst =
	let rec temp lst =
	match lst with
	| [] -> ""
	| a::l -> (let test = temp l in
		  if test = ""
		  then
		    a
		  else
		    a^test
		  )
	and temp2 lst =
	match lst with
	| [] -> []
	| a::l -> (let test = temp l in
		  if test = ""
		  then
		    [a]@(temp2 l)
		  else
		    [a^"."^test]@(temp2 l)
		  )
	in
	temp2 lst

   method print =
      let rec parcours tree name =
      match tree with
      | [] -> ()
      | Dns.DomainName (str, lst1, lst2)::l -> 
	 print_string "Name: ";
	 print_string (str^"."^name^"\n");
	 if lst2 != []
	 then
	    (print_string "Rdata: ";
	    List.iter Dnsprint.print_rdata lst2);
	 if l != []
	 then
	    (parcours l name);
	 if lst1 != []
	 then
            (print_string "-_-_-_-_-_-\n";
	    parcours lst1 (str^"."^name));
      in
      parcours root ""

      method private xts x =
	 match x with
	 | Dns.NS a -> a
	 | _ -> ""

      method private xta x =
	 match x with
	 | Dns.A a -> a
	 | _ -> "" 

end;;

let test = new dns_resolver in
test#query "www.lanetcie.com";
print_string "_-_-_-_\n";
test#query "www.yahoo.com";
;;
